/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"
#include <termios.h>

#define INPUT_UP (1 << 0)
#define INPUT_DOWN (1 << 1)
#define INPUT_LEFT (1 << 2)
#define INPUT_RIGHT (1 << 3)
#define INPUT_ACTION (1 << 4)
#define INPUT_CANCEL (1 << 5)
#define INPUT_COUNT (INPUT_CANCEL + 1)

#define INPUT_BUFFER_SIZE 128

typedef struct {
  uint8_t current;
  uint8_t previous;
} input_t;

extern input_t INPUT;

/**
 * Initializes the input system.
 */
void inputInit();

/**
 * Updates the input system.
 */
void inputUpdate();

/**
 * Disposes of the input system, restoring terminal settings.
 */
void inputDispose();

/**
 * Returns true if the specified key is currently pressed down.
 * 
 * @param key The key to check.
 * @return True if the key is down, false otherwise.
 */
bool_t inputIsDown(const uint8_t key);

/**
 * Returns true if the specified key was pressed down in the previous frame.
 * 
 * @param key The key to check.
 * @return True if the key was down in the previous frame, false otherwise.
 */
bool_t inputWasDown(const uint8_t key);

/**
 * Returns true if the specified key was pressed in the current frame.
 * 
 * @param key The key to check.
 * @return True if the key was pressed, false otherwise.
 */ 
bool_t inputWasPressed(const uint8_t key);

/**
 * Returns true if the specified key was released in the current frame.
 * 
 * @param key The key to check.
 * @return True if the key was released, false otherwise.
 */
bool_t inputWasReleased(const uint8_t key);