/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "entity.h"
#include "assert/assert.h"
#include "input.h"
#include "rpg/world/map.h"

void playerInit(entity_t *player) {
  assertNotNull(player, "Player entity is NULL");
  assertTrue(player->type == ENTITY_TYPE_PLAYER, "Entity is not a player");
}

void playerUpdate(entity_t *entity) {
  entity_t *other;
  assertNotNull(entity, "Entity is NULL");
  assertTrue(entity->type == ENTITY_TYPE_PLAYER, "Entity is not a player");

  // Handle movement
  if(entityIsWalking(entity)) {
    return;
  }

  entitydir_t dir = 0xFF;
  if(inputIsDown(INPUT_UP)) {
    dir = ENTITY_DIR_UP;
  } else if(inputIsDown(INPUT_DOWN)) {
    dir = ENTITY_DIR_DOWN;
  } else if(inputIsDown(INPUT_LEFT)) {
    dir = ENTITY_DIR_LEFT;
  } else if(inputIsDown(INPUT_RIGHT)) {
    dir = ENTITY_DIR_RIGHT;
  }
  
  if(dir != 0xFF) {
    if(dir != entity->dir) {
      entityTurn(entity, dir);
    } else {
      if(!entityCanWalk(entity, dir, &other)) return;
      entityWalk(entity);
    }
    return;
  }

  // Handle interaction
  if(inputWasPressed(INPUT_ACTION)) {
    int8_t dX, dY;
    entityDirGetPosition(entity->dir, 1, &dX, &dY);
    other = mapGetEntityAt(entity->x + dX, entity->y + dY);

    assertTrue(other != entity, "Player trying to interact with itself?");
    if(other != NULL && entityInteract(other, entity)) return;
  }
}