/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "map.h"
#include "util/memory.h"
#include "assert/assert.h"

map_t MAP;

void mapSet(const mapinfo_t mapInfo) {
  assertTrue(mapInfo.init != NULL, "Map initialization function cannot be NULL");
  mapInfo.init();
}

void mapInit(const uint8_t width, const uint8_t height) {
  memoryZero(&MAP, sizeof(map_t));

  MAP.width = width;
  MAP.height = height;
}

void mapUpdate() {
  entity_t *ent = MAP.entities;
  do {
    if(ent->type != ENTITY_TYPE_NULL) {
      entityUpdate(ent);
    }
    ent++;
  } while(ent < MAP.entities + MAP_ENTITY_COUNT);
}

entity_t * mapGetEntityAt(const uint8_t x, const uint8_t y) {
  if(x >= MAP.width || y >= MAP.height) return NULL;
  entity_t *ent = MAP.entities;
  do {
    if(ent->type != ENTITY_TYPE_NULL && ent->x == x && ent->y == y) return ent;
    ent++;
  } while(ent < MAP.entities + MAP_ENTITY_COUNT);

  return NULL;
}