/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "rpg/entity/entity.h"

#define MAP_ENTITY_COUNT 16

typedef struct {
  uint8_t width;
  uint8_t height;
  entity_t entities[MAP_ENTITY_COUNT];
} map_t;

typedef struct {
  const char *name;
  void (*init)(void);
} mapinfo_t;

extern map_t MAP;

/**
 * Sets the current map to the specified map information.
 * 
 * This function initializes the map with the provided map information,
 * including its name and initialization function.
 * 
 * @param mapInfo Map information containing the name and initialization function.
 */
void mapSet(const mapinfo_t mapInfo);

/**
 * Initializes the map with the given width and height.
 * 
 * @param width Width of the map.
 * @param height Height of the map.
 */
void mapInit(const uint8_t width, const uint8_t height);

/**
 * Updates the map, processing all entities.
 * 
 * This function should be called every frame to update the state of the map
 * and its entities.
 */
void mapUpdate();

/**
 * Gets the entity at the specified coordinates.
 * 
 * @param x X coordinate of the entity.
 * @param y Y coordinate of the entity.
 * @return Pointer to the entity at the specified coordinates or NULL.
 */
entity_t * mapGetEntityAt(const uint8_t x, const uint8_t y);